@@ -6,9 +6,11 @@ use std::path::Path;
 use std::sync::mpsc::{channel, Sender};
 use std::thread;
 
-use hyper::server::conn::AddrStream;
-use hyper::service::{make_service_fn, service_fn};
-use hyper::{Body, Request};
+use http_body_util::Full;
+use hyper::body::Bytes;
+use hyper::server::conn::http1;
+use hyper::service::service_fn;
+use hyper::Request;
 use tempfile::TempDir;
 
 pub fn tmp_dir() -> TempDir {
@@ -31,26 +33,38 @@ pub fn write_file(path: &Path, contents: &str) {
     file.sync_data().expect("writing test data");
 }
 
+// A dead simple hyper server implementation.
+// For more info, see:
+// https://hyper.rs/guides/1/server/hello-world/
 async fn run_server(addr_tx: Sender<SocketAddr>, addr: SocketAddr, contents: Vec<u8>) {
-    let make_svc = make_service_fn(|_: &AddrStream| {
+    let svc = service_fn(move |req: Request<hyper::body::Incoming>| {
         let contents = contents.clone();
         async move {
-            Ok::<_, Infallible>(service_fn(move |req: Request<Body>| {
-                let contents = contents.clone();
-                async move {
-                    let res = serve_contents(req, contents);
-                    Ok::<_, Infallible>(res)
-                }
-            }))
+            let res = serve_contents(req, contents);
+            Ok::<_, Infallible>(res)
         }
     });
 
-    let server = hyper::server::Server::bind(&addr).serve(make_svc);
-    let addr = server.local_addr();
+    let listener = tokio::net::TcpListener::bind(&addr)
+        .await
+        .expect("can not bind");
+
+    let addr = listener.local_addr().unwrap();
     addr_tx.send(addr).unwrap();
 
-    if let Err(e) = server.await {
-        eprintln!("server error: {e}");
+    loop {
+        let (stream, _) = listener
+            .accept()
+            .await
+            .expect("could not accept connection");
+        let io = hyper_util::rt::TokioIo::new(stream);
+
+        let svc = svc.clone();
+        tokio::spawn(async move {
+            if let Err(err) = http1::Builder::new().serve_connection(io, svc).await {
+                eprintln!("failed to serve connection: {:?}", err);
+            }
+        });
     }
 }
 
@@ -69,9 +83,9 @@ pub fn serve_file(contents: Vec<u8>) -> SocketAddr {
 }
 
 fn serve_contents(
-    req: hyper::Request<hyper::Body>,
+    req: hyper::Request<hyper::body::Incoming>,
     contents: Vec<u8>,
-) -> hyper::Response<hyper::Body> {
+) -> hyper::Response<Full<Bytes>> {
     let mut range_header = None;
     let (status, body) = if let Some(range) = req.headers().get(hyper::header::RANGE) {
         // extract range "bytes={start}-"
@@ -95,7 +109,7 @@ fn serve_contents(
     let mut res = hyper::Response::builder()
         .status(status)
         .header(hyper::header::CONTENT_LENGTH, body.len())
-        .body(hyper::Body::from(body))
+        .body(Full::new(Bytes::from(body)))
         .unwrap();
     if let Some(range) = range_header {
         res.headers_mut()
