@@ -1,5 +1,7 @@
 //! Just a dumping ground for cli stuff
 
+use std::cell::RefCell;
+use std::fmt::Display;
 use std::fs;
 use std::io::{BufRead, ErrorKind, Write};
 use std::path::Path;
@@ -8,23 +10,31 @@ use std::{cmp, env};
 
 use anyhow::{anyhow, Context, Result};
 use git_testament::{git_testament, render_testament};
-use lazy_static::lazy_static;
-use term2::Terminal;
+use once_cell::sync::Lazy;
 
 use super::self_update;
-use super::term2;
-use crate::dist::notifications as dist_notifications;
-use crate::process;
-use crate::toolchain::DistributableToolchain;
+use crate::cli::download_tracker::DownloadTracker;
+use crate::currentprocess::{
+    argsource::ArgSource,
+    filesource::{StdinSource, StdoutSource},
+    terminalsource,
+    varsource::VarSource,
+};
+use crate::dist::dist::{TargetTriple, ToolchainDesc};
+use crate::install::UpdateStatus;
 use crate::utils::notifications as util_notifications;
 use crate::utils::notify::NotificationLevel;
 use crate::utils::utils;
-use crate::{Cfg, Notification, Toolchain, UpdateStatus};
+use crate::{
+    dist::notifications as dist_notifications, toolchain::distributable::DistributableToolchain,
+};
+use crate::{process, toolchain::toolchain::Toolchain};
+use crate::{Cfg, Notification};
 
 pub(crate) const WARN_COMPLETE_PROFILE: &str = "downloading with complete profile isn't recommended unless you are a developer of the rust language";
 
 pub(crate) fn confirm(question: &str, default: bool) -> Result<bool> {
-    write!(process().stdout(), "{question} ")?;
+    write!(process().stdout().lock(), "{question} ")?;
     let _ = std::io::stdout().flush();
     let input = read_line()?;
 
@@ -35,7 +45,7 @@ pub(crate) fn confirm(question: &str, default: bool) -> Result<bool> {
         _ => false,
     };
 
-    writeln!(process().stdout())?;
+    writeln!(process().stdout().lock())?;
 
     Ok(r)
 }
@@ -46,12 +56,16 @@ pub(crate) enum Confirm {
     Advanced,
 }
 
-pub(crate) fn confirm_advanced() -> Result<Confirm> {
-    writeln!(process().stdout())?;
-    writeln!(process().stdout(), "1) Proceed with installation (default)")?;
-    writeln!(process().stdout(), "2) Customize installation")?;
-    writeln!(process().stdout(), "3) Cancel installation")?;
-    write!(process().stdout(), ">")?;
+pub(crate) fn confirm_advanced(customized_install: bool) -> Result<Confirm> {
+    writeln!(process().stdout().lock())?;
+    let first_option = match customized_install {
+        true => "1) Proceed with selected options (default - just press enter)",
+        false => "1) Proceed with standard installation (default - just press enter)",
+    };
+    writeln!(process().stdout().lock(), "{first_option}")?;
+    writeln!(process().stdout().lock(), "2) Customize installation")?;
+    writeln!(process().stdout().lock(), "3) Cancel installation")?;
+    write!(process().stdout().lock(), ">")?;
 
     let _ = std::io::stdout().flush();
     let input = read_line()?;
@@ -62,17 +76,17 @@ pub(crate) fn confirm_advanced() -> Result<Confirm> {
         _ => Confirm::No,
     };
 
-    writeln!(process().stdout())?;
+    writeln!(process().stdout().lock())?;
 
     Ok(r)
 }
 
 pub(crate) fn question_str(question: &str, default: &str) -> Result<String> {
-    writeln!(process().stdout(), "{question} [{default}]")?;
+    writeln!(process().stdout().lock(), "{question} [{default}]")?;
     let _ = std::io::stdout().flush();
     let input = read_line()?;
 
-    writeln!(process().stdout())?;
+    writeln!(process().stdout().lock())?;
 
     if input.is_empty() {
         Ok(default.to_string())
@@ -83,12 +97,12 @@ pub(crate) fn question_str(question: &str, default: &str) -> Result<String> {
 
 pub(crate) fn question_bool(question: &str, default: bool) -> Result<bool> {
     let default_text = if default { "(Y/n)" } else { "(y/N)" };
-    writeln!(process().stdout(), "{question} {default_text}")?;
+    writeln!(process().stdout().lock(), "{question} {default_text}")?;
 
     let _ = std::io::stdout().flush();
     let input = read_line()?;
 
-    writeln!(process().stdout())?;
+    writeln!(process().stdout().lock())?;
 
     if input.is_empty() {
         Ok(default)
@@ -156,19 +170,16 @@ impl NotifyOnConsole {
     }
 }
 
+#[cfg_attr(feature = "otel", tracing::instrument)]
 pub(crate) fn set_globals(verbose: bool, quiet: bool) -> Result<Cfg> {
-    use std::cell::RefCell;
-
-    use super::download_tracker::DownloadTracker;
-
-    let download_tracker = RefCell::new(DownloadTracker::new().with_display_progress(!quiet));
+    let download_tracker = DownloadTracker::new_with_display_progress(!quiet);
     let console_notifier = RefCell::new(NotifyOnConsole {
         verbose,
         ..Default::default()
     });
 
     Cfg::from_env(Arc::new(move |n: Notification<'_>| {
-        if download_tracker.borrow_mut().handle_notification(&n) {
+        if download_tracker.lock().unwrap().handle_notification(&n) {
             return;
         }
         console_notifier.borrow_mut().handle(n);
@@ -177,78 +188,97 @@ pub(crate) fn set_globals(verbose: bool, quiet: bool) -> Result<Cfg> {
 
 pub(crate) fn show_channel_update(
     cfg: &Cfg,
-    name: &str,
+    name: PackageUpdate,
     updated: Result<UpdateStatus>,
 ) -> Result<()> {
-    show_channel_updates(cfg, vec![(name.to_string(), updated)])
+    show_channel_updates(cfg, vec![(name, updated)])
 }
 
-fn show_channel_updates(cfg: &Cfg, toolchains: Vec<(String, Result<UpdateStatus>)>) -> Result<()> {
-    let data = toolchains.into_iter().map(|(name, result)| {
-        let toolchain = cfg.get_toolchain(&name, false)?;
-        let mut version: String = toolchain.rustc_version();
+pub(crate) enum PackageUpdate {
+    Rustup,
+    Toolchain(ToolchainDesc),
+}
 
-        let banner;
-        let color;
-        let mut previous_version: Option<String> = None;
-        match result {
-            Ok(UpdateStatus::Installed) => {
-                banner = "installed";
-                color = Some(term2::color::GREEN);
-            }
-            Ok(UpdateStatus::Updated(v)) => {
-                if name == "rustup" {
-                    previous_version = Some(env!("CARGO_PKG_VERSION").into());
-                    version = v;
-                } else {
-                    previous_version = Some(v);
-                }
-                banner = "updated";
-                color = Some(term2::color::GREEN);
-            }
-            Ok(UpdateStatus::Unchanged) => {
-                if name == "rustup" {
-                    version = env!("CARGO_PKG_VERSION").into();
-                }
-                banner = "unchanged";
-                color = None;
-            }
-            Err(_) => {
-                banner = "update failed";
-                color = Some(term2::color::RED);
-            }
+impl Display for PackageUpdate {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        match self {
+            PackageUpdate::Rustup => write!(f, "rustup"),
+            PackageUpdate::Toolchain(t) => write!(f, "{t}"),
         }
+    }
+}
 
-        let width = name.len() + 1 + banner.len();
+fn show_channel_updates(
+    cfg: &Cfg,
+    updates: Vec<(PackageUpdate, Result<UpdateStatus>)>,
+) -> Result<()> {
+    let data = updates.into_iter().map(|(pkg, result)| {
+        let (banner, color) = match &result {
+            Ok(UpdateStatus::Installed) => ("installed", Some(terminalsource::Color::Green)),
+            Ok(UpdateStatus::Updated(_)) => ("updated", Some(terminalsource::Color::Green)),
+            Ok(UpdateStatus::Unchanged) => ("unchanged", None),
+            Err(_) => ("update failed", Some(terminalsource::Color::Red)),
+        };
 
-        Ok((name, banner, width, color, version, previous_version))
+        let (previous_version, version) = match &pkg {
+            PackageUpdate::Rustup => {
+                let previous_version: Option<String> = match result {
+                    Ok(UpdateStatus::Installed) | Ok(UpdateStatus::Unchanged) | Err(_) => None,
+                    _ => Some(env!("CARGO_PKG_VERSION").into()),
+                };
+                let version = match result {
+                    Err(_) | Ok(UpdateStatus::Installed) | Ok(UpdateStatus::Unchanged) => {
+                        env!("CARGO_PKG_VERSION").into()
+                    }
+                    Ok(UpdateStatus::Updated(v)) => v,
+                };
+                (previous_version, version)
+            }
+            PackageUpdate::Toolchain(name) => {
+                // this is a bit strange: we don't supply the version we
+                // presumably had (for Installed and Unchanged), so we query it
+                // again. Perhaps we can do better.
+                let version = match Toolchain::new(cfg, name.into()) {
+                    Ok(t) => t.rustc_version(),
+                    Err(_) => String::from("(toolchain not installed)"),
+                };
+                let previous_version: Option<String> = match result {
+                    Ok(UpdateStatus::Installed) | Ok(UpdateStatus::Unchanged) | Err(_) => None,
+                    Ok(UpdateStatus::Updated(v)) => Some(v),
+                };
+                (previous_version, version)
+            }
+        };
+
+        let width = pkg.to_string().len() + 1 + banner.len();
+
+        Ok((pkg, banner, width, color, version, previous_version))
     });
 
-    let mut t = term2::stdout();
+    let mut t = process().stdout().terminal();
 
     let data: Vec<_> = data.collect::<Result<_>>()?;
     let max_width = data
         .iter()
         .fold(0, |a, &(_, _, width, _, _, _)| cmp::max(a, width));
 
-    for (name, banner, width, color, version, previous_version) in data {
+    for (pkg, banner, width, color, version, previous_version) in data {
         let padding = max_width - width;
         let padding: String = " ".repeat(padding);
-        let _ = write!(t, "  {padding}");
-        let _ = t.attr(term2::Attr::Bold);
+        let _ = write!(t.lock(), "  {padding}");
+        let _ = t.attr(terminalsource::Attr::Bold);
         if let Some(color) = color {
             let _ = t.fg(color);
         }
-        let _ = write!(t, "{name} ");
-        let _ = write!(t, "{banner}");
+        let _ = write!(t.lock(), "{pkg} {banner}");
         let _ = t.reset();
-        let _ = write!(t, " - {version}");
+        let _ = write!(t.lock(), " - {version}");
         if let Some(previous_version) = previous_version {
-            let _ = write!(t, " (from {previous_version})");
+            let _ = write!(t.lock(), " (from {previous_version})");
         }
-        let _ = writeln!(t);
+        let _ = writeln!(t.lock());
     }
-    let _ = writeln!(t);
+    let _ = writeln!(t.lock());
 
     Ok(())
 }
@@ -266,9 +296,13 @@ pub(crate) fn update_all_channels(
 
     let show_channel_updates = || {
         if !toolchains.is_empty() {
-            writeln!(process().stdout())?;
+            writeln!(process().stdout().lock())?;
 
-            show_channel_updates(cfg, toolchains)?;
+            let t = toolchains
+                .into_iter()
+                .map(|(p, s)| (PackageUpdate::Toolchain(p), s))
+                .collect();
+            show_channel_updates(cfg, t)?;
         }
         Ok(utils::ExitCode(0))
     };
@@ -341,10 +375,12 @@ where
     Ok(utils::ExitCode(0))
 }
 
-pub(crate) fn list_targets(toolchain: &Toolchain<'_>) -> Result<utils::ExitCode> {
-    let mut t = term2::stdout();
-    let distributable = DistributableToolchain::new_for_components(toolchain)?;
-    let components = distributable.list_components()?;
+pub(crate) fn list_targets(distributable: DistributableToolchain<'_>) -> Result<utils::ExitCode> {
+    let mut t = process().stdout().terminal();
+    let manifestation = distributable.get_manifestation()?;
+    let config = manifestation.read_config()?.unwrap_or_default();
+    let manifest = distributable.get_manifest()?;
+    let components = manifest.query_components(distributable.desc(), &config)?;
     for component in components {
         if component.component.short_name_in_manifest() == "rust-std" {
             let target = component
@@ -353,11 +389,11 @@ pub(crate) fn list_targets(toolchain: &Toolchain<'_>) -> Result<utils::ExitCode>
                 .as_ref()
                 .expect("rust-std should have a target");
             if component.installed {
-                let _ = t.attr(term2::Attr::Bold);
-                let _ = writeln!(t, "{target} (installed)");
+                let _ = t.attr(terminalsource::Attr::Bold);
+                let _ = writeln!(t.lock(), "{target} (installed)");
                 let _ = t.reset();
             } else if component.available {
-                let _ = writeln!(t, "{target}");
+                let _ = writeln!(t.lock(), "{target}");
             }
         }
     }
@@ -365,10 +401,14 @@ pub(crate) fn list_targets(toolchain: &Toolchain<'_>) -> Result<utils::ExitCode>
     Ok(utils::ExitCode(0))
 }
 
-pub(crate) fn list_installed_targets(toolchain: &Toolchain<'_>) -> Result<utils::ExitCode> {
-    let mut t = term2::stdout();
-    let distributable = DistributableToolchain::new_for_components(toolchain)?;
-    let components = distributable.list_components()?;
+pub(crate) fn list_installed_targets(
+    distributable: DistributableToolchain<'_>,
+) -> Result<utils::ExitCode> {
+    let t = process().stdout();
+    let manifestation = distributable.get_manifestation()?;
+    let config = manifestation.read_config()?.unwrap_or_default();
+    let manifest = distributable.get_manifest()?;
+    let components = manifest.query_components(distributable.desc(), &config)?;
     for component in components {
         if component.component.short_name_in_manifest() == "rust-std" {
             let target = component
@@ -377,41 +417,44 @@ pub(crate) fn list_installed_targets(toolchain: &Toolchain<'_>) -> Result<utils:
                 .as_ref()
                 .expect("rust-std should have a target");
             if component.installed {
-                writeln!(t, "{target}")?;
+                writeln!(t.lock(), "{target}")?;
             }
         }
     }
     Ok(utils::ExitCode(0))
 }
 
-pub(crate) fn list_components(toolchain: &Toolchain<'_>) -> Result<utils::ExitCode> {
-    let mut t = term2::stdout();
-    let distributable = DistributableToolchain::new_for_components(toolchain)?;
-    let components = distributable.list_components()?;
+pub(crate) fn list_components(
+    distributable: DistributableToolchain<'_>,
+) -> Result<utils::ExitCode> {
+    let mut t = process().stdout().terminal();
+
+    let manifestation = distributable.get_manifestation()?;
+    let config = manifestation.read_config()?.unwrap_or_default();
+    let manifest = distributable.get_manifest()?;
+    let components = manifest.query_components(distributable.desc(), &config)?;
     for component in components {
         let name = component.name;
         if component.installed {
-            t.attr(term2::Attr::Bold)?;
-            writeln!(t, "{name} (installed)")?;
+            t.attr(terminalsource::Attr::Bold)?;
+            writeln!(t.lock(), "{name} (installed)")?;
             t.reset()?;
         } else if component.available {
-            writeln!(t, "{name}")?;
+            writeln!(t.lock(), "{name}")?;
         }
     }
 
     Ok(utils::ExitCode(0))
 }
 
-pub(crate) fn list_installed_components(toolchain: &Toolchain<'_>) -> Result<utils::ExitCode> {
-    let mut t = term2::stdout();
-    let distributable = DistributableToolchain::new_for_components(toolchain)?;
-    let components = distributable.list_components()?;
-    for component in components {
+pub(crate) fn list_installed_components(distributable: DistributableToolchain<'_>) -> Result<()> {
+    let t = process().stdout();
+    for component in distributable.components()? {
         if component.installed {
-            writeln!(t, "{}", component.name)?;
+            writeln!(t.lock(), "{}", component.name)?;
         }
     }
-    Ok(utils::ExitCode(0))
+    Ok(())
 }
 
 fn print_toolchain_path(
@@ -421,11 +464,7 @@ fn print_toolchain_path(
     if_override: &str,
     verbose: bool,
 ) -> Result<()> {
-    let toolchain_path = {
-        let mut t_path = cfg.toolchains_dir.clone();
-        t_path.push(toolchain);
-        t_path
-    };
+    let toolchain_path = cfg.toolchains_dir.join(toolchain);
     let toolchain_meta = fs::symlink_metadata(&toolchain_path)?;
     let toolchain_path = if verbose {
         if toolchain_meta.is_dir() {
@@ -437,7 +476,7 @@ fn print_toolchain_path(
         String::new()
     };
     writeln!(
-        process().stdout(),
+        process().stdout().lock(),
         "{}{}{}{}",
         &toolchain,
         if_default,
@@ -448,35 +487,42 @@ fn print_toolchain_path(
 }
 
 pub(crate) fn list_toolchains(cfg: &Cfg, verbose: bool) -> Result<utils::ExitCode> {
-    let toolchains = cfg.list_toolchains()?;
+    // Work with LocalToolchainName to accommodate path based overrides
+    let toolchains = cfg
+        .list_toolchains()?
+        .iter()
+        .map(Into::into)
+        .collect::<Vec<_>>();
     if toolchains.is_empty() {
-        writeln!(process().stdout(), "no installed toolchains")?;
+        writeln!(process().stdout().lock(), "no installed toolchains")?;
     } else {
-        let def_toolchain_name = if let Ok(Some(def_toolchain)) = cfg.find_default() {
-            def_toolchain.name().to_string()
-        } else {
-            String::new()
-        };
+        let def_toolchain_name = cfg.get_default()?.map(|t| (&t).into());
         let cwd = utils::current_dir()?;
         let ovr_toolchain_name = if let Ok(Some((toolchain, _reason))) = cfg.find_override(&cwd) {
-            toolchain.name().to_string()
+            Some(toolchain)
         } else {
-            String::new()
+            None
         };
         for toolchain in toolchains {
-            let if_default = if def_toolchain_name == *toolchain {
+            let if_default = if def_toolchain_name.as_ref() == Some(&toolchain) {
                 " (default)"
             } else {
                 ""
             };
-            let if_override = if ovr_toolchain_name == *toolchain {
+            let if_override = if ovr_toolchain_name.as_ref() == Some(&toolchain) {
                 " (override)"
             } else {
                 ""
             };
 
-            print_toolchain_path(cfg, &toolchain, if_default, if_override, verbose)
-                .context("Failed to list toolchains' directories")?;
+            print_toolchain_path(
+                cfg,
+                &toolchain.to_string(),
+                if_default,
+                if_override,
+                verbose,
+            )
+            .context("Failed to list toolchains' directories")?;
         }
     }
     Ok(utils::ExitCode(0))
@@ -486,7 +532,7 @@ pub(crate) fn list_overrides(cfg: &Cfg) -> Result<utils::ExitCode> {
     let overrides = cfg.settings_file.with(|s| Ok(s.overrides.clone()))?;
 
     if overrides.is_empty() {
-        writeln!(process().stdout(), "no overrides")?;
+        writeln!(process().stdout().lock(), "no overrides")?;
     } else {
         let mut any_not_exist = false;
         for (k, v) in overrides {
@@ -495,7 +541,7 @@ pub(crate) fn list_overrides(cfg: &Cfg) -> Result<utils::ExitCode> {
                 any_not_exist = true;
             }
             writeln!(
-                process().stdout(),
+                process().stdout().lock(),
                 "{:<40}\t{:<20}",
                 utils::format_path_for_display(&k)
                     + if dir_exists { "" } else { " (not a directory)" },
@@ -503,7 +549,7 @@ pub(crate) fn list_overrides(cfg: &Cfg) -> Result<utils::ExitCode> {
             )?
         }
         if any_not_exist {
-            writeln!(process().stdout())?;
+            writeln!(process().stdout().lock())?;
             info!(
                 "you may remove overrides for non-existent directories with
 `rustup override unset --nonexistent`"
@@ -516,55 +562,61 @@ pub(crate) fn list_overrides(cfg: &Cfg) -> Result<utils::ExitCode> {
 git_testament!(TESTAMENT);
 
 pub(crate) fn version() -> &'static str {
-    lazy_static! {
-        // Because we trust our `stable` branch given the careful release
-        // process, we mark it trusted here so that our version numbers look
-        // right when built from CI before the tag is pushed
-        static ref RENDERED: String = render_testament!(TESTAMENT, "stable");
-    }
+    // Because we trust our `stable` branch given the careful release
+    // process, we mark it trusted here so that our version numbers look
+    // right when built from CI before the tag is pushed
+    static RENDERED: Lazy<String> = Lazy::new(|| render_testament!(TESTAMENT, "stable"));
     &RENDERED
 }
 
 pub(crate) fn dump_testament() -> Result<utils::ExitCode> {
     use git_testament::GitModification::*;
     writeln!(
-        process().stdout(),
+        process().stdout().lock(),
         "Rustup version renders as: {}",
         version()
     )?;
     writeln!(
-        process().stdout(),
+        process().stdout().lock(),
         "Current crate version: {}",
         env!("CARGO_PKG_VERSION")
     )?;
     if TESTAMENT.branch_name.is_some() {
         writeln!(
-            process().stdout(),
+            process().stdout().lock(),
             "Built from branch: {}",
             TESTAMENT.branch_name.unwrap()
         )?;
     } else {
-        writeln!(process().stdout(), "Branch information missing")?;
+        writeln!(process().stdout().lock(), "Branch information missing")?;
     }
-    writeln!(process().stdout(), "Commit info: {}", TESTAMENT.commit)?;
+    writeln!(
+        process().stdout().lock(),
+        "Commit info: {}",
+        TESTAMENT.commit
+    )?;
     if TESTAMENT.modifications.is_empty() {
-        writeln!(process().stdout(), "Working tree is clean")?;
+        writeln!(process().stdout().lock(), "Working tree is clean")?;
     } else {
         for fmod in TESTAMENT.modifications {
             match fmod {
-                Added(f) => writeln!(process().stdout(), "Added: {}", String::from_utf8_lossy(f))?,
+                Added(f) => writeln!(
+                    process().stdout().lock(),
+                    "Added: {}",
+                    String::from_utf8_lossy(f)
+                )?,
                 Removed(f) => writeln!(
-                    process().stdout(),
+                    process().stdout().lock(),
                     "Removed: {}",
                     String::from_utf8_lossy(f)
                 )?,
                 Modified(f) => writeln!(
-                    process().stdout(),
+                    process().stdout().lock(),
                     "Modified: {}",
                     String::from_utf8_lossy(f)
                 )?,
                 Untracked(f) => writeln!(
-                    process().stdout(),
+                    process().stdout().lock(),
                     "Untracked: {}",
                     String::from_utf8_lossy(f)
                 )?,
@@ -617,3 +669,14 @@ pub(crate) fn ignorable_error(error: &'static str, no_prompt: bool) -> Result<()
         Err(error)
     }
 }
+
+/// Warns if rustup is running under emulation, such as macOS Rosetta
+pub(crate) fn warn_if_host_is_emulated() {
+    if TargetTriple::is_host_emulated() {
+        warn!(
+            "Rustup is not running natively. It's running under emulation of {}.",
+            TargetTriple::from_host_or_build()
+        );
+        warn!("For best compatibility and performance you should reinstall rustup for your native CPU.");
+    }
+}
