@@ -1,13 +1,11 @@
 use std::collections::VecDeque;
 use std::fmt;
 use std::io::Write;
+use std::sync::{Arc, Mutex};
 use std::time::{Duration, Instant};
 
-use term::Terminal;
-
-use super::term2;
+use crate::currentprocess::{filesource::StdoutSource, process, terminalsource};
 use crate::dist::Notification as In;
-use crate::utils::tty;
 use crate::utils::units::{Size, Unit, UnitMode};
 use crate::utils::Notification as Un;
 use crate::Notification;
@@ -16,6 +14,9 @@ use crate::Notification;
 const DOWNLOAD_TRACK_COUNT: usize = 5;
 
 /// Tracks download progress and displays information about it to a terminal.
+///
+/// *not* safe for tracking concurrent downloads yet - it is basically undefined
+/// what will happen.
 pub(crate) struct DownloadTracker {
     /// Content-Length of the to-be downloaded object.
     content_len: Option<usize>,
@@ -31,11 +32,7 @@ pub(crate) struct DownloadTracker {
     last_sec: Option<Instant>,
     /// Time stamp of the start of the download
     start_sec: Option<Instant>,
-    /// The terminal we write the information to.
-    /// XXX: Could be a term trait, but with #1818 on the horizon that
-    ///      is a pointless change to make - better to let that transition
-    ///      happen and take stock after that.
-    term: term2::StdoutTerminal,
+    term: terminalsource::ColorableTerminal,
     /// Whether we displayed progress for the download or not.
     ///
     /// If the download is quick enough, we don't have time to
@@ -53,24 +50,19 @@ pub(crate) struct DownloadTracker {
 
 impl DownloadTracker {
     /// Creates a new DownloadTracker.
-    pub(crate) fn new() -> Self {
-        Self {
+    pub(crate) fn new_with_display_progress(display_progress: bool) -> Arc<Mutex<Self>> {
+        Arc::new(Mutex::new(Self {
             content_len: None,
             total_downloaded: 0,
             downloaded_this_sec: 0,
             downloaded_last_few_secs: VecDeque::with_capacity(DOWNLOAD_TRACK_COUNT),
             start_sec: None,
             last_sec: None,
-            term: term2::stdout(),
+            term: process().stdout().terminal(),
             displayed_charcount: None,
             units: vec![Unit::B],
-            display_progress: true,
-        }
-    }
-
-    pub(crate) fn with_display_progress(mut self, display_progress: bool) -> Self {
-        self.display_progress = display_progress;
-        self
+            display_progress,
+        }))
     }
 
     pub(crate) fn handle_notification(&mut self, n: &Notification<'_>) -> bool {
@@ -81,7 +73,7 @@ impl DownloadTracker {
                 true
             }
             Notification::Install(In::Utils(Un::DownloadDataReceived(data))) => {
-                if tty::stdout_isatty() {
+                if process().stdout().is_a_tty() {
                     self.data_received(data.len());
                 }
                 true
@@ -139,7 +131,7 @@ impl DownloadTracker {
         if self.displayed_charcount.is_some() {
             // Display the finished state
             self.display();
-            let _ = writeln!(self.term);
+            let _ = writeln!(self.term.lock());
         }
         self.prepare_for_new_download();
     }
@@ -178,8 +170,8 @@ impl DownloadTracker {
                     // This is not ideal as very narrow terminals might mess up,
                     // but it is more likely to succeed until term's windows console
                     // fixes whatever's up with delete_line().
-                    let _ = write!(self.term, "{}", " ".repeat(n));
-                    let _ = self.term.flush();
+                    let _ = write!(self.term.lock(), "{}", " ".repeat(n));
+                    let _ = self.term.lock().flush();
                     let _ = self.term.carriage_return();
                 }
 
@@ -211,9 +203,9 @@ impl DownloadTracker {
                     ),
                 };
 
-                let _ = write!(self.term, "{output}");
+                let _ = write!(self.term.lock(), "{output}");
                 // Since stdout is typically line-buffered and we don't print a newline, we manually flush.
-                let _ = self.term.flush();
+                let _ = self.term.lock().flush();
                 self.displayed_charcount = Some(output.chars().count());
             }
         }
@@ -268,7 +260,10 @@ fn format_dhms(sec: u64) -> (u64, u8, u8, u8) {
 
 #[cfg(test)]
 mod tests {
+    use rustup_macros::unit_test as test;
+
     use super::format_dhms;
+
     #[test]
     fn download_tracker_format_dhms_test() {
         assert_eq!(format_dhms(2), (0, 0, 0, 2));
