@@ -1,4 +1,4 @@
-pub(crate) static RUSTUP_HELP: &str = r"DISCUSSION:
+pub(crate) static RUSTUP_HELP: &str = r"Discussion:
     Rustup installs The Rust Programming Language from the official
     release channels, enabling you to easily switch between stable,
     beta, and nightly compilers and keep them updated. It makes
@@ -8,7 +8,7 @@ pub(crate) static RUSTUP_HELP: &str = r"DISCUSSION:
     If you are new to Rust consider running `rustup doc --book` to
     learn Rust.";
 
-pub(crate) static SHOW_HELP: &str = r"DISCUSSION:
+pub(crate) static SHOW_HELP: &str = r"Discussion:
     Shows the name of the active toolchain and the version of `rustc`.
 
     If the active toolchain has installed support for additional
@@ -17,7 +17,7 @@ pub(crate) static SHOW_HELP: &str = r"DISCUSSION:
     If there are multiple toolchains installed then all installed
     toolchains are listed as well.";
 
-pub(crate) static SHOW_ACTIVE_TOOLCHAIN_HELP: &str = r"DISCUSSION:
+pub(crate) static SHOW_ACTIVE_TOOLCHAIN_HELP: &str = r"Discussion:
     Shows the name of the active toolchain.
 
     This is useful for figuring out the active tool chain from
@@ -26,7 +26,7 @@ pub(crate) static SHOW_ACTIVE_TOOLCHAIN_HELP: &str = r"DISCUSSION:
     You should use `rustc --print sysroot` to get the sysroot, or
     `rustc --version` to get the toolchain version.";
 
-pub(crate) static UPDATE_HELP: &str = r"DISCUSSION:
+pub(crate) static UPDATE_HELP: &str = r"Discussion:
     With no toolchain specified, the `update` command updates each of
     the installed toolchains from the official release channels, then
     updates rustup itself.
@@ -34,22 +34,22 @@ pub(crate) static UPDATE_HELP: &str = r"DISCUSSION:
     If given a toolchain argument then `update` updates that
     toolchain, the same as `rustup toolchain install`.";
 
-pub(crate) static INSTALL_HELP: &str = r"DISCUSSION:
+pub(crate) static INSTALL_HELP: &str = r"Discussion:
     Installs a specific rust toolchain.
 
     The 'install' command is an alias for 'rustup update <toolchain>'.";
 
-pub(crate) static DEFAULT_HELP: &str = r"DISCUSSION:
+pub(crate) static DEFAULT_HELP: &str = r"Discussion:
     Sets the default toolchain to the one specified. If the toolchain
     is not already installed then it is installed first.";
 
-pub(crate) static TOOLCHAIN_HELP: &str = r"DISCUSSION:
+pub(crate) static TOOLCHAIN_HELP: &str = r"Discussion:
     Many `rustup` commands deal with *toolchains*, a single
     installation of the Rust compiler. `rustup` supports multiple
     types of toolchains. The most basic track the official release
     channels: 'stable', 'beta' and 'nightly'; but `rustup` can also
-    install toolchains from the official archives, for alternate host
-    platforms, and from local builds.
+    install specific toolchains from the official archives, toolchains for
+    alternate host platforms, and from local builds ('custom toolchains').
 
     Standard release channel toolchain names have the following form:
 
@@ -85,12 +85,14 @@ pub(crate) static TOOLCHAIN_HELP: &str = r"DISCUSSION:
     often used for developing Rust itself. For more information see
     `rustup toolchain help link`.";
 
-pub(crate) static TOOLCHAIN_LINK_HELP: &str = r"DISCUSSION:
+pub(crate) static TOOLCHAIN_LINK_HELP: &str = r"Discussion:
     'toolchain' is the custom name to be assigned to the new toolchain.
-    Any name is permitted as long as it does not fully match an initial
-    substring of a standard release channel. For example, you can use
-    the names 'latest' or '2017-04-01' but you cannot use 'stable' or
-    'beta-i686' or 'nightly-x86_64-unknown-linux-gnu'.
+    Any name is permitted as long as:
+    - it does not include '/' or '\' except as the last character
+    - it is not equal to 'none'
+    - it does not fully match an initialsubstring of a standard release channel.
+    For example, you can use the names 'latest' or '2017-04-01' but you cannot
+    use 'stable' or 'beta-i686' or 'nightly-x86_64-unknown-linux-gnu'.
 
     'path' specifies the directory where the binaries and libraries for
     the custom toolchain can be found. For example, when used for
@@ -104,7 +106,7 @@ pub(crate) static TOOLCHAIN_LINK_HELP: &str = r"DISCUSSION:
     If you now compile a crate in the current directory, the custom
     toolchain 'latest-stage1' will be used.";
 
-pub(crate) static OVERRIDE_HELP: &str = r"DISCUSSION:
+pub(crate) static OVERRIDE_HELP: &str = r"Discussion:
     Overrides configure Rustup to use a specific toolchain when
     running in a specific directory.
 
@@ -125,14 +127,14 @@ pub(crate) static OVERRIDE_HELP: &str = r"DISCUSSION:
     override and use the default toolchain again, `rustup override
     unset`.";
 
-pub(crate) static OVERRIDE_UNSET_HELP: &str = r"DISCUSSION:
+pub(crate) static OVERRIDE_UNSET_HELP: &str = r"Discussion:
     If `--path` argument is present, removes the override toolchain
     for the specified directory. If `--nonexistent` argument is
     present, removes the override toolchain for all nonexistent
     directories. Otherwise, removes the override toolchain for the
     current directory.";
 
-pub(crate) static RUN_HELP: &str = r"DISCUSSION:
+pub(crate) static RUN_HELP: &str = r"Discussion:
     Configures an environment to use the given toolchain and then runs
     the specified program. The command may be any program, not just
     rustc or cargo. This can be used for testing arbitrary toolchains
@@ -147,14 +149,14 @@ pub(crate) static RUN_HELP: &str = r"DISCUSSION:
 
         $ rustup run nightly cargo build";
 
-pub(crate) static DOC_HELP: &str = r"DISCUSSION:
+pub(crate) static DOC_HELP: &str = r"Discussion:
     Opens the documentation for the currently active toolchain with
     the default browser.
 
     By default, it opens the documentation index. Use the various
     flags to open specific pieces of documentation.";
 
-pub(crate) static COMPLETIONS_HELP: &str = r"DISCUSSION:
+pub(crate) static COMPLETIONS_HELP: &str = r"Discussion:
     Enable tab completion for Bash, Fish, Zsh, or PowerShell
     The script is output on `stdout`, allowing one to re-direct the
     output to the file of their choosing. Where you place the file
@@ -165,7 +167,7 @@ pub(crate) static COMPLETIONS_HELP: &str = r"DISCUSSION:
     Here are some common set ups for the three supported shells under
     Unix and similar operating systems (such as GNU/Linux).
 
-    BASH:
+    Bash:
 
     Completion files are commonly stored in `/etc/bash_completion.d/` for
     system-wide commands, but can be stored in
@@ -178,7 +180,7 @@ pub(crate) static COMPLETIONS_HELP: &str = r"DISCUSSION:
     This installs the completion script. You may have to log out and
     log back in to your shell session for the changes to take effect.
 
-    BASH (macOS/Homebrew):
+    Bash (macOS/Homebrew):
 
     Homebrew stores bash completion files within the Homebrew directory.
     With the `bash-completion` brew formula installed, run the command:
@@ -186,7 +188,7 @@ pub(crate) static COMPLETIONS_HELP: &str = r"DISCUSSION:
         $ mkdir -p $(brew --prefix)/etc/bash_completion.d
         $ rustup completions bash > $(brew --prefix)/etc/bash_completion.d/rustup.bash-completion
 
-    FISH:
+    Fish:
 
     Fish completion files are commonly stored in
     `$HOME/.config/fish/completions`. Run the command:
@@ -197,7 +199,7 @@ pub(crate) static COMPLETIONS_HELP: &str = r"DISCUSSION:
     This installs the completion script. You may have to log out and
     log back in to your shell session for the changes to take effect.
 
-    ZSH:
+    Zsh:
 
     ZSH completions are commonly stored in any directory listed in
     your `$fpath` variable. To use these completions, you must either
@@ -227,7 +229,7 @@ pub(crate) static COMPLETIONS_HELP: &str = r"DISCUSSION:
 
     for the new completions to take effect.
 
-    CUSTOM LOCATIONS:
+    Custom locations:
 
     Alternatively, you could save these files to the place of your
     choosing, such as a custom directory inside your $HOME. Doing so
@@ -235,7 +237,7 @@ pub(crate) static COMPLETIONS_HELP: &str = r"DISCUSSION:
     inside your login script. Consult your shells documentation for
     how to add such directives.
 
-    POWERSHELL:
+    PowerShell:
 
     The powershell completion scripts require PowerShell v5.0+ (which
     comes with Windows 10, but can be downloaded separately for windows 7
@@ -259,24 +261,34 @@ pub(crate) static COMPLETIONS_HELP: &str = r"DISCUSSION:
 
         PS C:\> rustup completions powershell >> ${env:USERPROFILE}\Documents\WindowsPowerShell\Microsoft.PowerShell_profile.ps1
 
-    CARGO:
+    Cargo:
 
     Rustup can also generate a completion script for `cargo`. The script output
     by `rustup` will source the completion script distributed with your default
     toolchain. Not all shells are currently supported. Here are examples for
     the currently supported shells.
 
-    BASH:
+    Bash:
 
         $ rustup completions bash cargo >> ~/.local/share/bash-completion/completions/cargo
 
-    ZSH:
+    Zsh:
 
         $ rustup completions zsh cargo > ~/.zfunc/_cargo";
 
-pub(crate) static TOOLCHAIN_ARG_HELP: &str = "Toolchain name, such as 'stable', 'nightly', \
+pub(crate) static OFFICIAL_TOOLCHAIN_ARG_HELP: &str =
+    "Toolchain name, such as 'stable', 'nightly', \
                                        or '1.8.0'. For more information see `rustup \
                                        help toolchain`";
+pub(crate) static RESOLVABLE_LOCAL_TOOLCHAIN_ARG_HELP: &str = "Toolchain name, such as 'stable', 'nightly', \
+                                       '1.8.0', or a custom toolchain name, or an absolute path. For more \
+                                       information see `rustup help toolchain`";
+pub(crate) static RESOLVABLE_TOOLCHAIN_ARG_HELP: &str = "Toolchain name, such as 'stable', 'nightly', \
+                                       '1.8.0', or a custom toolchain name. For more information see `rustup \
+                                       help toolchain`";
+pub(crate) static MAYBE_RESOLVABLE_TOOLCHAIN_ARG_HELP: &str = "'none', a toolchain name, such as 'stable', 'nightly', \
+                                       '1.8.0', or a custom toolchain name. For more information see `rustup \
+                                       help toolchain`";
 
 pub(crate) static TOPIC_ARG_HELP: &str = "Topic such as 'core', 'fn', 'usize', 'eprintln!', \
                                    'core::arch', 'alloc::format!', 'std::fs', \
