@@ -1007,8 +1007,8 @@ fn update(cfg: &mut Cfg, m: &ArgMatches) -> Result<utils::ExitCode> {
     } else {
         common::update_all_channels(cfg, self_update, m.get_flag("force"))?;
         info!("cleaning up downloads & tmp directories");
-        utils::delete_dir_contents(&cfg.download_dir);
-        cfg.temp_cfg.clean();
+        utils::delete_dir_contents_following_links(&cfg.download_dir);
+        cfg.tmp_cx.clean();
     }
 
     if !self_update::NEVER_SELF_UPDATE && self_update_mode == SelfUpdateMode::CheckOnly {
@@ -1379,8 +1379,7 @@ fn component_add(cfg: &Cfg, m: &ArgMatches) -> Result<utils::ExitCode> {
     let target = get_target(m, &distributable);
 
     for component in m.get_many::<String>("component").unwrap() {
-        let new_component = Component::new_with_target(component, false)
-            .unwrap_or_else(|| Component::new(component.to_string(), target.clone(), true));
+        let new_component = Component::try_new(component, &distributable, target.as_ref())?;
         distributable.add_component(new_component)?;
     }
 
@@ -1400,8 +1399,7 @@ fn component_remove(cfg: &Cfg, m: &ArgMatches) -> Result<utils::ExitCode> {
     let target = get_target(m, &distributable);
 
     for component in m.get_many::<String>("component").unwrap() {
-        let new_component = Component::new_with_target(component, false)
-            .unwrap_or_else(|| Component::new(component.to_string(), target.clone(), true));
+        let new_component = Component::try_new(component, &distributable, target.as_ref())?;
         distributable.remove_component(new_component)?;
     }
 
@@ -1594,18 +1592,15 @@ fn doc(cfg: &Cfg, m: &ArgMatches) -> Result<utils::ExitCode> {
     };
 
     let topical_path: PathBuf;
+    let mut doc_name = m.get_one::<String>("topic").map(|s| s.as_str());
 
-    let doc_url = if let Some(topic) = m.get_one::<String>("topic") {
+    let doc_url = if let Some(topic) = doc_name {
         topical_path = topical_doc::local_path(&toolchain.doc_path("").unwrap(), topic)?;
         topical_path.to_str().unwrap()
     } else if let Some((name, _, path)) = DOCS_DATA.iter().find(|(name, _, _)| m.get_flag(name)) {
-        writeln!(
-            process().stderr().lock(),
-            "Opening docs named `{name}` in your browser"
-        )?;
+        doc_name = Some(name);
         path
     } else {
-        writeln!(process().stderr().lock(), "Opening docs in your browser")?;
         "index.html"
     };
 
@@ -1614,6 +1609,14 @@ fn doc(cfg: &Cfg, m: &ArgMatches) -> Result<utils::ExitCode> {
         writeln!(process().stdout().lock(), "{}", doc_path.display())?;
         Ok(utils::ExitCode(0))
     } else {
+        if let Some(name) = doc_name {
+            writeln!(
+                process().stderr().lock(),
+                "Opening docs named `{name}` in your browser"
+            )?;
+        } else {
+            writeln!(process().stderr().lock(), "Opening docs in your browser")?;
+        }
         toolchain.open_docs(doc_url)?;
         Ok(utils::ExitCode(0))
     }
