@@ -79,7 +79,7 @@ use crate::{
 };
 
 use os::*;
-pub(crate) use os::{delete_rustup_and_cargo_home, run_update, self_replace};
+pub(crate) use os::{run_update, self_replace};
 #[cfg(windows)]
 pub use windows::complete_windows_uninstall;
 
@@ -132,14 +132,13 @@ impl FromStr for SelfUpdateMode {
     }
 }
 
-impl ToString for SelfUpdateMode {
-    fn to_string(&self) -> String {
-        match self {
+impl std::fmt::Display for SelfUpdateMode {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        f.write_str(match self {
             SelfUpdateMode::Enable => "enable",
             SelfUpdateMode::Disable => "disable",
             SelfUpdateMode::CheckOnly => "check-only",
-        }
-        .into()
+        })
     }
 }
 
@@ -332,7 +331,14 @@ These components can be acquired through a Visual Studio installer.
 
 "#;
 
-static UPDATE_ROOT: &str = "https://static.rust-lang.org/rustup";
+static DEFAULT_UPDATE_ROOT: &str = "https://static.rust-lang.org/rustup";
+
+fn update_root() -> String {
+    process()
+        .var("RUSTUP_UPDATE_ROOT")
+        .inspect(|url| debug!("`RUSTUP_UPDATE_ROOT` has been set to `{url}`"))
+        .unwrap_or_else(|_| String::from(DEFAULT_UPDATE_ROOT))
+}
 
 /// `CARGO_HOME` suitable for display, possibly with $HOME
 /// substituted for the directory prefix
@@ -1100,12 +1106,9 @@ pub(crate) fn update(cfg: &Cfg) -> Result<utils::ExitCode> {
 
     match prepare_update()? {
         Some(setup_path) => {
-            let version = match get_new_rustup_version(&setup_path) {
-                Some(new_version) => parse_new_rustup_version(new_version),
-                None => {
-                    err!("failed to get rustup version");
-                    return Ok(utils::ExitCode(1));
-                }
+            let Some(version) = get_and_parse_new_rustup_version(&setup_path) else {
+                err!("failed to get rustup version");
+                return Ok(utils::ExitCode(1));
             };
 
             let _ = common::show_channel_update(
@@ -1129,6 +1132,10 @@ pub(crate) fn update(cfg: &Cfg) -> Result<utils::ExitCode> {
     Ok(utils::ExitCode(0))
 }
 
+fn get_and_parse_new_rustup_version(path: &Path) -> Option<String> {
+    get_new_rustup_version(path).map(parse_new_rustup_version)
+}
+
 fn get_new_rustup_version(path: &Path) -> Option<String> {
     match Command::new(path).arg("--version").output() {
         Err(_) => None,
@@ -1171,7 +1178,7 @@ pub(crate) fn prepare_update() -> Result<Option<PathBuf>> {
 
     // For windows x86 builds seem slow when used with windows defender.
     // The website defaulted to i686-windows-gnu builds for a long time.
-    // This ensures that we update to a version thats appropriate for users
+    // This ensures that we update to a version that's appropriate for users
     // and also works around if the website messed up the detection.
     // If someone really wants to use another version, they still can enforce
     // that using the environment variable RUSTUP_OVERRIDE_HOST_TRIPLE.
@@ -1179,9 +1186,7 @@ pub(crate) fn prepare_update() -> Result<Option<PathBuf>> {
     let triple = dist::TargetTriple::from_host().unwrap_or(triple);
 
     // Get update root.
-    let update_root = process()
-        .var("RUSTUP_UPDATE_ROOT")
-        .unwrap_or_else(|_| String::from(UPDATE_ROOT));
+    let update_root = update_root();
 
     // Get current version
     let current_version = env!("CARGO_PKG_VERSION");
@@ -1212,9 +1217,7 @@ pub(crate) fn prepare_update() -> Result<Option<PathBuf>> {
 }
 
 pub(crate) fn get_available_rustup_version() -> Result<String> {
-    let update_root = process()
-        .var("RUSTUP_UPDATE_ROOT")
-        .unwrap_or_else(|_| String::from(UPDATE_ROOT));
+    let update_root = update_root();
     let tempdir = tempfile::Builder::new()
         .prefix("rustup-update")
         .tempdir()
