@@ -500,13 +500,23 @@ pub(crate) fn do_remove_from_path() -> Result<()> {
 
 const RUSTUP_UNINSTALL_ENTRY: &str = r"Software\Microsoft\Windows\CurrentVersion\Uninstall\Rustup";
 
+fn rustup_uninstall_reg_key() -> Result<RegKey> {
+    Ok(RegKey::predef(HKEY_CURRENT_USER)
+        .create_subkey(RUSTUP_UNINSTALL_ENTRY)
+        .context("Failed creating uninstall key")?
+        .0)
+}
+
+pub(crate) fn do_update_programs_display_version(version: &str) -> Result<()> {
+    rustup_uninstall_reg_key()?
+        .set_value("DisplayVersion", &version)
+        .context("Failed to set `DisplayVersion`")
+}
+
 pub(crate) fn do_add_to_programs() -> Result<()> {
     use std::path::PathBuf;
 
-    let key = RegKey::predef(HKEY_CURRENT_USER)
-        .create_subkey(RUSTUP_UNINSTALL_ENTRY)
-        .context("Failed creating uninstall key")?
-        .0;
+    let key = rustup_uninstall_reg_key()?;
 
     // Don't overwrite registry if Rustup is already installed
     let prev = key
@@ -531,14 +541,11 @@ pub(crate) fn do_add_to_programs() -> Result<()> {
         vtype: RegType::REG_SZ,
     };
 
-    let current_version: &str = env!("CARGO_PKG_VERSION");
-
     key.set_raw_value("UninstallString", &reg_value)
-        .context("Failed to set uninstall string")?;
+        .context("Failed to set `UninstallString`")?;
     key.set_value("DisplayName", &"Rustup: the Rust toolchain installer")
-        .context("Failed to set display name")?;
-    key.set_value("DisplayVersion", &current_version)
-        .context("Failed to set display version")?;
+        .context("Failed to set `DisplayName`")?;
+    do_update_programs_display_version(env!("CARGO_PKG_VERSION"))?;
 
     Ok(())
 }
@@ -586,6 +593,12 @@ pub(crate) fn run_update(setup_path: &Path) -> Result<utils::ExitCode> {
         .spawn()
         .context("unable to run updater")?;
 
+    let Some(version) = super::get_and_parse_new_rustup_version(setup_path) else {
+        warn!("failed to get the new rustup version in order to update `DisplayVersion`");
+        return Ok(utils::ExitCode(1));
+    };
+    do_update_programs_display_version(&version)?;
+
     Ok(utils::ExitCode(0))
 }
 
