@@ -1,5 +1,5 @@
 use std::borrow::Cow;
-use std::fmt::{self, Display};
+use std::fmt::{self, Debug, Display};
 use std::io;
 use std::path::{Path, PathBuf};
 use std::process::Command;
@@ -7,7 +7,6 @@ use std::str::FromStr;
 use std::sync::Arc;
 
 use anyhow::{anyhow, bail, Context, Result};
-use derivative::Derivative;
 use serde::Deserialize;
 use thiserror::Error as ThisError;
 
@@ -172,8 +171,6 @@ impl OverrideCfg {
 
 pub(crate) const UNIX_FALLBACK_SETTINGS: &str = "/etc/rustup/settings.toml";
 
-#[derive(Derivative)]
-#[derivative(Debug)]
 pub(crate) struct Cfg {
     profile_override: Option<dist::Profile>,
     pub rustup_dir: PathBuf,
@@ -182,11 +179,10 @@ pub(crate) struct Cfg {
     pub toolchains_dir: PathBuf,
     pub update_hash_dir: PathBuf,
     pub download_dir: PathBuf,
-    pub temp_cfg: temp::Cfg,
+    pub tmp_cx: temp::Context,
     pub toolchain_override: Option<ResolvableToolchainName>,
     pub env_override: Option<LocalToolchainName>,
     pub dist_root_url: String,
-    #[derivative(Debug = "ignore")]
     pub notify_handler: Arc<dyn Fn(Notification<'_>)>,
 }
 
@@ -230,13 +226,17 @@ impl Cfg {
             .transpose()?;
 
         let dist_root_server = match process().var("RUSTUP_DIST_SERVER") {
-            Ok(ref s) if !s.is_empty() => s.clone(),
+            Ok(s) if !s.is_empty() => {
+                debug!("`RUSTUP_DIST_SERVER` has been set to `{s}`");
+                s
+            }
             _ => {
                 // For backward compatibility
                 process()
                     .var("RUSTUP_DIST_ROOT")
                     .ok()
                     .and_then(utils::if_not_empty)
+                    .inspect(|url| debug!("`RUSTUP_DIST_ROOT` has been set to `{url}`"))
                     .map_or(Cow::Borrowed(dist::DEFAULT_DIST_ROOT), Cow::Owned)
                     .as_ref()
                     .trim_end_matches("/dist")
@@ -245,7 +245,7 @@ impl Cfg {
         };
 
         let notify_clone = notify_handler.clone();
-        let temp_cfg = temp::Cfg::new(
+        let tmp_cx = temp::Context::new(
             rustup_dir.join("tmp"),
             dist_root_server.as_str(),
             Box::new(move |n| (notify_clone)(n.into())),
@@ -260,7 +260,7 @@ impl Cfg {
             toolchains_dir,
             update_hash_dir,
             download_dir,
-            temp_cfg,
+            tmp_cx,
             notify_handler,
             toolchain_override: None,
             env_override,
@@ -285,7 +285,7 @@ impl Cfg {
     ) -> DownloadCfg<'a> {
         DownloadCfg {
             dist_root: &self.dist_root_url,
-            temp_cfg: &self.temp_cfg,
+            tmp_cx: &self.tmp_cx,
             download_dir: &self.download_dir,
             notify_handler,
         }
@@ -428,7 +428,7 @@ impl Cfg {
                 }
 
                 self.settings_file.with_mut(|s| {
-                    s.version = DEFAULT_METADATA_VERSION.to_owned();
+                    DEFAULT_METADATA_VERSION.clone_into(&mut s.version);
                     Ok(())
                 })
             }
@@ -590,8 +590,6 @@ impl Cfg {
                             // disabling this and backing out https://github.com/rust-lang/rustup/pull/2141 (but provide
                             // the base name in the error to help users)
                             let resolved_name = &ToolchainName::try_from(toolchain_name_str)?;
-                            let ts = self.list_toolchains()?;
-                            eprintln!("{resolved_name:?} {ts:?}");
                             if !self.list_toolchains()?.iter().any(|s| s == resolved_name) {
                                 return Err(anyhow!(format!(
                                     "target triple in channel name '{name}'"
@@ -908,7 +906,7 @@ impl Cfg {
         // breadcrumb in case of regression: we used to get the cargo path and
         // cargo.exe path separately, not using the binary_file helper. This may
         // matter if calling a binary with some personality that allows .exe and
-        // not .exe to coexist (e.g. wine) - but thats not something we aim to
+        // not .exe to coexist (e.g. wine) - but that's not something we aim to
         // support : the host should always be correct.
         if cargo_path.exists() {
             return Ok(None);
@@ -957,6 +955,39 @@ impl Cfg {
     }
 }
 
+impl Debug for Cfg {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        let Self {
+            profile_override,
+            rustup_dir,
+            settings_file,
+            fallback_settings,
+            toolchains_dir,
+            update_hash_dir,
+            download_dir,
+            tmp_cx,
+            toolchain_override,
+            env_override,
+            dist_root_url,
+            notify_handler: _,
+        } = self;
+
+        f.debug_struct("Cfg")
+            .field("profile_override", profile_override)
+            .field("rustup_dir", rustup_dir)
+            .field("settings_file", settings_file)
+            .field("fallback_settings", fallback_settings)
+            .field("toolchains_dir", toolchains_dir)
+            .field("update_hash_dir", update_hash_dir)
+            .field("download_dir", download_dir)
+            .field("tmp_cx", tmp_cx)
+            .field("toolchain_override", toolchain_override)
+            .field("env_override", env_override)
+            .field("dist_root_url", dist_root_url)
+            .finish()
+    }
+}
+
 fn get_default_host_triple(s: &Settings) -> dist::TargetTriple {
     s.default_host_triple
         .as_ref()
