@@ -1,38 +1,49 @@
 /// Adapts currentprocess to the trait home::Env
 use std::ffi::OsString;
 use std::io;
-use std::ops::Deref;
 use std::path::PathBuf;
 
 use home::env as home;
 
-use super::CurrentDirSource;
-use super::HomeProcess;
 use super::OSProcess;
-use super::TestProcess;
-use super::VarSource;
+use super::Process;
+#[cfg(feature = "test")]
+use super::{CurrentDirSource, TestProcess, VarSource};
 
-impl home::Env for Box<dyn HomeProcess + 'static> {
+impl home::Env for Process {
     fn home_dir(&self) -> Option<PathBuf> {
-        (**self).home_dir()
+        match self {
+            Process::OSProcess(p) => p.home_dir(),
+            #[cfg(feature = "test")]
+            Process::TestProcess(p) => p.home_dir(),
+        }
     }
     fn current_dir(&self) -> Result<PathBuf, io::Error> {
-        home::Env::current_dir(&(**self))
+        match self {
+            Process::OSProcess(p) => home::Env::current_dir(p),
+            #[cfg(feature = "test")]
+            Process::TestProcess(p) => home::Env::current_dir(p),
+        }
     }
     fn var_os(&self, key: &str) -> Option<OsString> {
-        home::Env::var_os(&(**self), key)
+        match self {
+            Process::OSProcess(p) => home::Env::var_os(p, key),
+            #[cfg(feature = "test")]
+            Process::TestProcess(p) => home::Env::var_os(p, key),
+        }
     }
 }
 
+#[cfg(feature = "test")]
 impl home::Env for TestProcess {
     fn home_dir(&self) -> Option<PathBuf> {
         self.var("HOME").ok().map(|v| v.into())
     }
     fn current_dir(&self) -> Result<PathBuf, io::Error> {
-        CurrentDirSource::current_dir(self.deref())
+        CurrentDirSource::current_dir(self)
     }
     fn var_os(&self, key: &str) -> Option<OsString> {
-        VarSource::var_os(self.deref(), key)
+        VarSource::var_os(self, key)
     }
 }
 
