@@ -143,7 +143,7 @@ impl Settings {
     }
 
     pub(crate) fn stringify(self) -> String {
-        toml::Value::Table(self.into_toml()).to_string()
+        self.into_toml().to_string()
     }
 
     pub(crate) fn from_toml(mut table: toml::value::Table, path: &str) -> Result<Self> {
