@@ -7,9 +7,6 @@ use std::io::Write;
 use std::path::Path;
 use std::str;
 
-#[cfg(not(windows))]
-use libc;
-
 #[cfg(not(windows))]
 use crate::{currentprocess::varsource::VarSource, process};
 
@@ -34,7 +31,7 @@ pub fn is_file<P: AsRef<Path>>(path: P) -> bool {
 }
 
 #[cfg(windows)]
-pub fn open_dir(p: &Path) -> std::io::Result<File> {
+pub fn open_dir_following_links(p: &Path) -> std::io::Result<File> {
     use std::fs::OpenOptions;
     use std::os::windows::fs::OpenOptionsExt;
 
@@ -45,14 +42,13 @@ pub fn open_dir(p: &Path) -> std::io::Result<File> {
     options.custom_flags(FILE_FLAG_BACKUP_SEMANTICS);
     options.open(p)
 }
+
 #[cfg(not(windows))]
-pub fn open_dir(p: &Path) -> std::io::Result<File> {
+pub fn open_dir_following_links(p: &Path) -> std::io::Result<File> {
     use std::fs::OpenOptions;
-    use std::os::unix::fs::OpenOptionsExt;
 
     let mut options = OpenOptions::new();
     options.read(true);
-    options.custom_flags(libc::O_NOFOLLOW);
     options.open(p)
 }
 
@@ -119,7 +115,6 @@ pub(crate) fn filter_file<F: FnMut(&str) -> bool>(
 
 pub fn append_file(dest: &Path, line: &str) -> io::Result<()> {
     let mut dest_file = fs::OpenOptions::new()
-        .write(true)
         .append(true)
         .create(true)
         .open(dest)?;
