@@ -5,11 +5,12 @@ use std::io::Write;
 use std::process::Stdio;
 
 use rustup::for_host;
-use rustup::test::this_host_triple;
-use rustup::test::with_saved_path;
+use rustup::test::{
+    mock::clitools::{self, set_current_dist_date, Config, SanitizedOutput, Scenario},
+    this_host_triple, with_saved_path,
+};
 use rustup::utils::raw;
-
-use crate::mock::clitools::{self, set_current_dist_date, Config, SanitizedOutput, Scenario};
+use rustup_macros::integration_test as test;
 
 fn run_input(config: &Config, args: &[&str], input: &str) -> SanitizedOutput {
     run_input_with_env(config, args, input, &[])
@@ -89,7 +90,7 @@ Current installation options:
                profile: default
   modify PATH variable: no
 
-1) Proceed with installation (default)
+1) Proceed with standard installation (default - just press enter)
 2) Customize installation
 3) Cancel installation
 >
@@ -130,7 +131,11 @@ fn blank_lines_around_stderr_log_output_update() {
         config.expect_ok(&["rustup-init", "-y", "--no-modify-path"]);
         let out = run_input(
             config,
-            &["rustup-init", "--no-update-default-toolchain"],
+            &[
+                "rustup-init",
+                "--no-update-default-toolchain",
+                "--no-modify-path",
+            ],
             "\n\n",
         );
         println!("-- stdout --\n {}", out.stdout);
@@ -525,7 +530,7 @@ fn install_stops_if_rustc_exists() {
     clitools::test(Scenario::SimpleV2, &|config| {
         let out = config.run(
             "rustup-init",
-            &["--no-modify-path"],
+            ["--no-modify-path"],
             &[
                 ("RUSTUP_INIT_SKIP_PATH_CHECK", "no"),
                 ("PATH", temp_dir_path),
@@ -555,7 +560,7 @@ fn install_stops_if_cargo_exists() {
     clitools::test(Scenario::SimpleV2, &|config| {
         let out = config.run(
             "rustup-init",
-            &["--no-modify-path"],
+            ["--no-modify-path"],
             &[
                 ("RUSTUP_INIT_SKIP_PATH_CHECK", "no"),
                 ("PATH", temp_dir_path),
@@ -585,7 +590,7 @@ fn with_no_prompt_install_succeeds_if_rustc_exists() {
     clitools::test(Scenario::SimpleV2, &|config| {
         let out = config.run(
             "rustup-init",
-            &["-y", "--no-modify-path"],
+            ["-y", "--no-modify-path"],
             &[
                 ("RUSTUP_INIT_SKIP_PATH_CHECK", "no"),
                 ("PATH", temp_dir_path),
