@@ -541,6 +541,22 @@ fn run_rls_when_not_installed() {
     });
 }
 
+#[test]
+fn run_rls_when_not_installed_for_nightly() {
+    setup(&|config| {
+        config.expect_ok(&["rustup", "default", "stable"]);
+        config.expect_ok(&["rustup", "toolchain", "install", "nightly"]);
+        config.expect_err(
+            &["rls", "+nightly", "--version"],
+            &format!(
+                "'rls{}' is not installed for the toolchain 'nightly-{}'.\nTo install, run `rustup component add --toolchain nightly-{1} rls`",
+                EXE_SUFFIX,
+                this_host_triple(),
+            ),
+        );
+    });
+}
+
 #[test]
 fn run_rust_lldb_when_not_in_toolchain() {
     clitools::test(Scenario::UnavailableRls, &|config| {
@@ -661,7 +677,63 @@ fn rename_rls_remove() {
     });
 }
 
+// issue #3737
+/// `~/.rustup/toolchains` is permitted to be a symlink.
+#[test]
+#[cfg(any(unix, windows))]
+fn toolchains_symlink() {
+    use rustup::utils::raw::symlink_dir;
+    use std::fs;
+
+    clitools::test(Scenario::SimpleV2, &|config| {
+        let cwd = config.current_dir();
+        let test_toolchains = cwd.join("toolchains-test");
+        fs::create_dir(&test_toolchains).unwrap();
+        symlink_dir(&test_toolchains, &config.rustupdir.join("toolchains")).unwrap();
+
+        config.expect_ok(&["rustup", "default", "nightly"]);
+        config.expect_ok_contains(&["rustup", "toolchain", "list"], "nightly", "");
+        config.expect_ok_contains(&["rustc", "--version"], "hash-nightly-2", "");
+        config.expect_ok(&["rustup", "toolchain", "uninstall", "nightly"]);
+        config.expect_stdout_ok(
+            &["rustup", "toolchain", "list"],
+            "no installed toolchains\n",
+        );
+    });
+}
+
+// issue #3344
+/// `~/.rustup/tmp` and `~/.rustup/downloads` are permitted to be symlinks.
+#[test]
+#[cfg(any(unix, windows))]
+fn tmp_downloads_symlink() {
+    use rustup::utils::raw::symlink_dir;
+    use std::fs;
+
+    clitools::test(Scenario::ArchivesV2, &|config| {
+        let cwd = config.current_dir();
+
+        let test_tmp = cwd.join("tmp-test");
+        fs::create_dir(&test_tmp).unwrap();
+        symlink_dir(&test_tmp, &config.rustupdir.join("tmp")).unwrap();
+
+        let test_downloads = cwd.join("tmp-downloads");
+        fs::create_dir(&test_downloads).unwrap();
+        symlink_dir(&test_downloads, &config.rustupdir.join("downloads")).unwrap();
+
+        set_current_dist_date(config, "2015-01-01");
+        config.expect_ok(&["rustup", "default", "nightly"]);
+
+        set_current_dist_date(config, "2015-01-02");
+        config.expect_ok(&["rustup", "update"]);
+
+        assert!(config.rustupdir.join("tmp").exists());
+        assert!(config.rustupdir.join("downloads").exists());
+    });
+}
+
 // issue #1169
+/// A toolchain that is a stale symlink should be correctly uninstalled.
 #[test]
 #[cfg(any(unix, windows))]
 fn toolchain_broken_symlink() {
