@@ -10,7 +10,7 @@ use remove_dir_all::remove_dir_all;
 
 use retry::{
     delay::{jitter, Fibonacci},
-    retry, OperationResult,
+    retry,
 };
 use rustup::test::{
     mock::{
@@ -23,6 +23,9 @@ use rustup::utils::{raw, utils};
 use rustup::{for_host, DUP_TOOLS, TOOLS};
 use rustup_macros::integration_test as test;
 
+#[cfg(windows)]
+use rustup::test::with_saved_reg_value;
+
 const TEST_VERSION: &str = "1.1.1";
 
 pub fn update_setup(f: &dyn Fn(&mut Config, &Path)) {
@@ -267,26 +270,29 @@ fn uninstall_doesnt_leave_gc_file() {
         // 100ms, but during the contention of test suites can be substantially
         // longer while still succeeding.
 
-        #[derive(thiserror::Error, Debug)]
-        #[error("garbage remaining: {:?}", .0)]
-        struct GcErr(Vec<String>);
-
-        match retry(Fibonacci::from_millis(1).map(jitter).take(23), || {
-            let garbage = fs::read_dir(parent)
-                .unwrap()
-                .map(|d| d.unwrap().path().to_string_lossy().to_string())
-                .collect::<Vec<_>>();
-            match garbage.len() {
-                0 => OperationResult::Ok(()),
-                _ => OperationResult::Retry(GcErr(garbage)),
-            }
-        }) {
+        let check = || ensure_empty(parent);
+        match retry(Fibonacci::from_millis(1).map(jitter).take(23), check) {
             Ok(_) => (),
             Err(e) => panic!("{e}"),
         }
     })
 }
 
+fn ensure_empty(dir: &Path) -> Result<(), GcErr> {
+    let garbage = fs::read_dir(dir)
+        .unwrap()
+        .map(|d| d.unwrap().path().to_string_lossy().to_string())
+        .collect::<Vec<_>>();
+    match garbage.len() {
+        0 => Ok(()),
+        _ => Err(GcErr(garbage)),
+    }
+}
+
+#[derive(thiserror::Error, Debug)]
+#[error("garbage remaining: {:?}", .0)]
+struct GcErr(Vec<String>);
+
 #[test]
 fn update_exact() {
     let version = env!("CARGO_PKG_VERSION");
@@ -301,7 +307,40 @@ info: downloading self-update
             &["rustup", "self", "update"],
             &format!("  rustup updated - {version} (from {version})\n\n",),
             &expected_output,
-        )
+        );
+    });
+}
+
+#[test]
+#[cfg(windows)]
+fn update_overwrites_programs_display_version() {
+    let root = &winreg::RegKey::predef(winreg::enums::HKEY_CURRENT_USER);
+    let key = r"Software\Microsoft\Windows\CurrentVersion\Uninstall\Rustup";
+    let name = "DisplayVersion";
+
+    const PLACEHOLDER_VERSION: &str = "9.999.99";
+    let version = env!("CARGO_PKG_VERSION");
+
+    update_setup(&|config, _| {
+        with_saved_reg_value(root, key, name, &mut || {
+            config.expect_ok(&["rustup-init", "-y", "--no-modify-path"]);
+
+            root.create_subkey(key)
+                .unwrap()
+                .0
+                .set_value(name, &PLACEHOLDER_VERSION)
+                .unwrap();
+
+            config.expect_ok(&["rustup", "self", "update"]);
+
+            assert_eq!(
+                root.open_subkey(key)
+                    .unwrap()
+                    .get_value::<String, _>(name)
+                    .unwrap(),
+                version,
+            );
+        });
     });
 }
 
