@@ -111,13 +111,13 @@ fn basic_install() {
     let prefix = InstallPrefix::from(instdir.path().to_owned());
 
     let tmpdir = tempfile::Builder::new().prefix("rustup").tempdir().unwrap();
-    let tmpcfg = temp::Cfg::new(
+    let tmp_cx = temp::Context::new(
         tmpdir.path().to_owned(),
         DEFAULT_DIST_SERVER,
         Box::new(|_| ()),
     );
     let notify = |_: Notification<'_>| ();
-    let tx = Transaction::new(prefix.clone(), &tmpcfg, &notify);
+    let tx = Transaction::new(prefix.clone(), &tmp_cx, &notify);
 
     let components = Components::open(prefix).unwrap();
 
@@ -157,13 +157,13 @@ fn multiple_component_install() {
     let prefix = InstallPrefix::from(instdir.path().to_owned());
 
     let tmpdir = tempfile::Builder::new().prefix("rustup").tempdir().unwrap();
-    let tmpcfg = temp::Cfg::new(
+    let tmp_cx = temp::Context::new(
         tmpdir.path().to_owned(),
         DEFAULT_DIST_SERVER,
         Box::new(|_| ()),
     );
     let notify = |_: Notification<'_>| ();
-    let tx = Transaction::new(prefix.clone(), &tmpcfg, &notify);
+    let tx = Transaction::new(prefix.clone(), &tmp_cx, &notify);
 
     let components = Components::open(prefix).unwrap();
 
@@ -207,13 +207,13 @@ fn uninstall() {
     let prefix = InstallPrefix::from(instdir.path().to_owned());
 
     let tmpdir = tempfile::Builder::new().prefix("rustup").tempdir().unwrap();
-    let tmpcfg = temp::Cfg::new(
+    let tmp_cx = temp::Context::new(
         tmpdir.path().to_owned(),
         DEFAULT_DIST_SERVER,
         Box::new(|_| ()),
     );
     let notify = |_: Notification<'_>| ();
-    let tx = Transaction::new(prefix.clone(), &tmpcfg, &notify);
+    let tx = Transaction::new(prefix.clone(), &tmp_cx, &notify);
 
     let components = Components::open(prefix.clone()).unwrap();
 
@@ -225,7 +225,7 @@ fn uninstall() {
 
     // Now uninstall
     let notify = |_: Notification<'_>| ();
-    let mut tx = Transaction::new(prefix, &tmpcfg, &notify);
+    let mut tx = Transaction::new(prefix, &tmp_cx, &notify);
     for component in components.list().unwrap() {
         tx = component.uninstall(tx).unwrap();
     }
@@ -264,13 +264,13 @@ fn component_bad_version() {
     let prefix = InstallPrefix::from(instdir.path().to_owned());
 
     let tmpdir = tempfile::Builder::new().prefix("rustup").tempdir().unwrap();
-    let tmpcfg = temp::Cfg::new(
+    let tmp_cx = temp::Context::new(
         tmpdir.path().to_owned(),
         DEFAULT_DIST_SERVER,
         Box::new(|_| ()),
     );
     let notify = |_: Notification<'_>| ();
-    let tx = Transaction::new(prefix.clone(), &tmpcfg, &notify);
+    let tx = Transaction::new(prefix.clone(), &tmp_cx, &notify);
 
     let components = Components::open(prefix.clone()).unwrap();
 
@@ -310,13 +310,13 @@ fn install_to_prefix_that_does_not_exist() {
     let prefix = InstallPrefix::from(does_not_exist.clone());
 
     let tmpdir = tempfile::Builder::new().prefix("rustup").tempdir().unwrap();
-    let tmpcfg = temp::Cfg::new(
+    let tmp_cx = temp::Context::new(
         tmpdir.path().to_owned(),
         DEFAULT_DIST_SERVER,
         Box::new(|_| ()),
     );
     let notify = |_: Notification<'_>| ();
-    let tx = Transaction::new(prefix.clone(), &tmpcfg, &notify);
+    let tx = Transaction::new(prefix.clone(), &tmp_cx, &notify);
 
     let components = Components::open(prefix).unwrap();
 
